// Библиотека работы с устройствами по I2C
#include <Wire.h> 
// Библиотека работы с TSL2561
#include <Adafruit_Sensor.h>
#include <Adafruit_TSL2561_U.h>
#include <TimerOne.h>
#include "sav_button.h"

#include <U8glib.h>
U8GLIB_PCD8544 u8g(10, 4, 6, 5, 7);    // SPI Com: SCK = 10, MOSI = 4, CS = 6, A0 = 5, Reset = 7

 
#define MAX_UNITS  84 

Adafruit_TSL2561_Unified tsl = Adafruit_TSL2561_Unified(TSL2561_ADDR_FLOAT, 12345);
SButton button1(2,100,0,0,0);
SButton button2(3,100,0,0,0);

boolean DebugFlag = true;


int key_count          = 0; 
unsigned long key_time = 0;
boolean key_press      = false;
boolean power_on       = true;
boolean light_on       = true;

int key_count2          = 0; 
unsigned long key_time2 = 0;
boolean key_press2      = false;
int key_mode2           = 1;


volatile boolean int_pwr_up  = false;
volatile boolean int_key     = false;

// Callback функция по таймеру
uint8_t timer_loop = 0;
uint8_t sound_loop = 0;

// Callback функция по таймеру
void timerIsr()
{
   switch( button1.Loop()){
      case SB_CLICK : //Короткое нажатие
         light_on = !light_on;
         digitalWrite(9,light_on);
         break;
   }

   switch( button2.Loop()){
      case SB_CLICK : //Короткое нажатие
         if( key_mode2 == 1){
            key_mode2 = 2;
         }
         else {
            key_mode2 = 1;
         }
         break;
   }
}



/**
* Функция определения напряжения питания устройства
*/
static int vccRead(byte us =250) {
  ADMUX = 1<<REFS0; // опорное напряжение - Vcc
  ADMUX |= 0x0E;    // объект измерения - внутренний источник
                    // стабилизированного напряжения 1.1В
  delayMicroseconds(us);
 
  ADCSRA |= 1<<ADSC;         // запуск АЦ-преобразования
  while(ADCSRA & (1<<ADSC)); // и ожидание его завершения
  word x = ADC;
  return x ? (1100L * 1023) / x : -1;
}



void DebugPrint( char *msg ){
   if( DebugFlag )Serial.println(msg);
} 

uint16_t l1=0,l2=0,l_min=0,l_max=0;
uint16_t u1 = 0;
double pulse = 0;
uint16_t l_mas[MAX_UNITS];


/**
* Текстовый экран
*/
void DisplayMode1(){
   char s[20];
   // graphic commands to redraw the complete screen should be placed here  
   u8g.firstPage();  
   do {
 
      sprintf(s,"%u lx",l1);
      u8g.setFont(u8g_font_10x20);
      u8g.drawStr( 0, 20, s);
      u8g.setFont(u8g_font_6x10);
      sprintf(s,"Pulse: %d.%d%c",(int)pulse,(int)(pulse*10)%10,'%'); 
      u8g.drawStr( 0, 30, s);
      sprintf(s,"Bat: %d.%d V",u1/1000,u1%1000); 
      u8g.drawStr( 0, 39, s);
      u8g.drawStr( 0, 48, "Sensor TSL2561"); 
  } while( u8g.nextPage() );

 } 

/**
* Графический экран
*/
void DisplayMode2(){ 
   char s[20];
   // graphic commands to redraw the complete screen should be placed here  
   u8g.firstPage();  
   do {
 
      sprintf(s,"%u lx %d.%d%c",l1,(int)pulse,(int)(pulse*10)%10,'%');
      u8g.setFont(u8g_font_5x7);
      u8g.drawStr( 0, 7, s);
      
      uint16_t dl = l_max - l_min;
      u8g.drawLine(0, 45, 84, 45 );
      u8g.drawLine(78, 43, 84, 45 );
      u8g.drawLine(78, 47, 84, 45 );
      for( int i=0; i<MAX_UNITS; i++){
         uint16_t y = 44-(l_max-l_mas[i])*28/dl;
         u8g.drawLine(i, y, i, y-4);
      }  


  } while( u8g.nextPage() );

  
} 


 
void setup()
{
   if( DebugFlag ){
      Serial.begin(115200);
      Serial.println("Start luxmeter");
   }
   pinMode(13,OUTPUT);      // Светодиод питания
   pinMode(9,OUTPUT); // Выход на подсветку дисплея
   pinMode(8,OUTPUT); // Выход на питание дисплея
   digitalWrite(8,HIGH);
   digitalWrite(9,light_on);
   button1.begin();
   button2.begin();
   Timer1.initialize(10000);  
   Timer1.attachInterrupt( timerIsr ); 
   
//




 
}




 
unsigned long ms1 = 0; 
 
void loop()
{
   unsigned long ms = millis();

   
   
   
// Опрос датчиков каждую секунду
   if( (ms < ms1 || (ms - ms1) > 500)){
      ms1 = ms;
      l1=0;l2=0;l_min=0;l_max=0;
      u1 = vccRead(100);
      uint32_t l_avg = 0;
      sensors_event_t event;
// Делаем MSX_UNITS измерений по 13мс      
      for( int i=0; i<MAX_UNITS; i++){
         tsl.getEvent(&event);
         l_avg += event.light;
         l_mas[i] = event.light;
         if( i == 0 ){
            l_min = event.light;
            l_max = event.light;
         }
         else {
            if( event.light < l_min )l_min = event.light;
            if( event.light > l_max )l_max = event.light;
         }
      } 
      l1 = l_avg / MAX_UNITS;
      if( l_max != 0 )pulse = (double)((l_max - l_min))*100/(double)l_max;
      else pulse = 0; 
      if( DebugFlag ){
         Serial.print("l_min(lx)=");
         Serial.print(l_min);
         Serial.print(",l_max(lx)=");
         Serial.print(l_max);
         Serial.print(",l1(lx)=");
         Serial.print(l1);
         Serial.print(",pulse(%)=");
         Serial.print(pulse,1);
         Serial.print(",Vcc (mV)=");
         Serial.println(u1);
     }

     if( key_mode2 == 1)DisplayMode1();
     else DisplayMode2();
   }
 
}
